<?php
// CloudMatic Key Transfer
$workDir = '/tmp/cloudmatic';
$currentPath = __DIR__;

$hg = new Homegear\Homegear();

// create workdir
mkdir($workDir, 0700);
// check if this is the initial tansfer
if ($hg->getData('CloudMatic','transferPending') === true) {
    $snr = $hg->getData('CloudMatic','snr');
    // get keyfile
    keyDownload("https://www.cloudmatic.de/getmhkeyccu2.php?snr=$snr&tkey=$snr", $workDir);
    if (file_exists($workDir.'/key.tar.gz') && filesize($workDir.'/key.tar.gz') > 0) {
        // keyfile exists, extract the needed files
        exec(escapeshellcmd("tar -xzvf $workDir/key.tar.gz -C $workDir"),$result);
        $result = installFiles($hg,$workDir);
        if ($result == 1) {
            // everything worked fine ... set current version, remove pending & install new crontab for regular updates (twice a day)
            $lines = file($workDir.'/mhcfg');
            foreach ($lines as $line) {
                if (stristr($line,'userkennung')) {
                    $data = explode('=',$line);
                    $hg->setData('CloudMatic','username', trim($data[1]));
                }
                if (stristr($line,'userid')) {
                    $data = explode('=',$line);
                    $hg->setData('CloudMatic','userid', trim($data[1]));
                }
                if (stristr($line,'userkey')) {
                    $data = explode('=',$line);
                    $hg->setData('CloudMatic','userkey', trim($data[1]));
                }
            }
            $version = file($workDir.'/newver');
            $hg->setData('CloudMatic','Version',trim($version[0]));
            $hg->setData('CloudMatic','transferPending', false);
            if (stripos($currentPath,'/admin-ui/')) {
                // clear crontab and set new one for OSS
                exec('crontab -u homegear -r');
                $cronjob = fopen("/tmp/$snr", "w");
                fwrite($cronjob,"0 */6 * * * /usr/bin/perl -e 'sleep int rand 600'; /usr/bin/homegear -e rs ../admin-ui/bin/keytransfer.php > /dev/null 2>&1".PHP_EOL);
                fclose($cronjob);
                exec('crontab -u homegear /tmp/'.$snr);
            } else {
                // clear crontab
                exec('crontab -u homegear -r');
            }
            unlink('/tmp/'.$snr);
            // enable & start OpenVPN
            $hg->managementServiceCommand("openvpn@cloudmatic", 'enable');
            checkResult($hg);
            $hg->managementServiceCommand("openvpn@cloudmatic", 'start');
        }
    }
} else {
    // normal update
    $cmid = $hg->getData('CloudMatic','userid');
    $cmkey = $hg->getData('CloudMatic','userkey');
    if ($cmid == '' || $cmkey == '') { die('Fatal error: CloudMatic ID and Key not found!'); }
    keyDownload("https://www.cloudmatic.de/getkeyccu2.php?id=$cmid&key=$cmkey", $workDir);
    exec(escapeshellcmd("tar -xzvf $workDir/key.tar.gz -C $workDir"),$result);
    // check if key is newer
    $version = file($workDir.'/newver');
    if (trim($version[0]) > $hg->getData('CloudMatic','Version')) {
        // version is newer, install new files
        $result = installFiles($hg,$workDir);
        if ($result == 1) {
            // all fine, set new version & restart service
            $hg->setData('CloudMatic','Version',trim($version[0]));
            $hg->managementServiceCommand("openvpn@cloudmatic", 'restart');
        }   
    }
}
// clean up
exec(escapeshellcmd('rm -rf '.$workDir));

function keyDownload($url = false, $workDir) {
    if ($url === false) { return; }
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    $data = curl_exec($ch);
    $file = fopen($workDir.'/key.tar.gz','w');
    fputs($file,$data);
    fclose($file);
}

function installFiles($hg,$workDir) {
    $res = 0;
    // check if needed files exist
    if (file_exists($workDir.'/client.conf') && file_exists($workDir.'/client.crt') && file_exists($workDir.'/client.key') && file_exists($workDir.'/mhca.crt') && file_exists($workDir.'/mhcfg')) {
        // install needed files
        foreach (['client.conf', 'client.crt', 'client.key', 'mhca.crt', 'mhcfg'] as $file) {
            $data[$file] = file_get_contents($workDir.'/'.$file);
        }
        // adjust paths
        $data['client.conf'] = preg_replace('/\/usr\/local\/etc\/config\/addons\/mh/','/etc/openvpn/cloudmatic',$data['client.conf']);
        try {
            $hg->managementWriteCloudMaticConfig($data['client.conf'],$data['mhcfg'],$data['mhca.crt'],$data['client.crt'],$data['client.key']);
        } catch(\Homegear\HomegearException $e) {
            die('Exception catched. Code: '.$e->getCode().'. Message: '.$e->getMessage());
        }
        $res = 1;
    }
    return $res;
}
function checkResult($hg) {
    $result = $hg->managementGetCommandStatus();
    while ($result[0] == 256) {
        sleep(1);
        $result = $hg->managementGetCommandStatus();
    }
    return $result[0];
}